@@ -180,7 +180,6 @@ where
 
 type LargePrimitiveBuilder<T> = MutableListArray<i64, MutablePrimitiveArray<T>>;
 type LargeListUtf8Builder = MutableListArray<i64, MutableUtf8Array<i64>>;
-#[cfg(feature = "dtype-binary")]
 type LargeListBinaryBuilder = MutableListArray<i64, MutableBinaryArray<i64>>;
 type LargeListBooleanBuilder = MutableListArray<i64, MutableBooleanArray>;
 
@@ -264,14 +263,12 @@ impl ListBuilderTrait for ListUtf8ChunkedBuilder {
     }
 }
 
-#[cfg(feature = "dtype-binary")]
 pub struct ListBinaryChunkedBuilder {
     builder: LargeListBinaryBuilder,
     field: Field,
     fast_explode: bool,
 }
 
-#[cfg(feature = "dtype-binary")]
 impl ListBinaryChunkedBuilder {
     pub fn new(name: &str, capacity: usize, values_capacity: usize) -> Self {
         let values = MutableBinaryArray::<i64>::with_capacity(values_capacity);
@@ -317,7 +314,6 @@ impl ListBinaryChunkedBuilder {
     }
 }
 
-#[cfg(feature = "dtype-binary")]
 impl ListBuilderTrait for ListBinaryChunkedBuilder {
     fn append_opt_series(&mut self, opt_s: Option<&Series>) {
         match opt_s {
@@ -475,7 +471,6 @@ pub fn get_list_builder(
                     Box::new(builder)
                 }};
             }
-            #[cfg(feature = "dtype-binary")]
             macro_rules! get_binary_builder {
                 () => {{
                     let builder =
@@ -541,6 +536,7 @@ impl<'a> AnonymousListBuilder<'a> {
 
     #[inline]
     pub fn append_null(&mut self) {
+        self.fast_explode = false;
         self.builder.push_null();
     }
 
@@ -570,6 +566,7 @@ impl<'a> AnonymousListBuilder<'a> {
     }
 
     pub fn finish(&mut self) -> ListChunked {
+        // don't use self from here on one
         let slf = std::mem::take(self);
         if slf.builder.is_empty() {
             ListChunked::full_null_with_dtype(&slf.name, 0, &slf.dtype.unwrap_or(DataType::Null))
@@ -577,9 +574,9 @@ impl<'a> AnonymousListBuilder<'a> {
             let dtype = slf.dtype.map(|dt| dt.to_physical().to_arrow());
             let arr = slf.builder.finish(dtype.as_ref()).unwrap();
             let dtype = DataType::from(arr.data_type());
-            let mut ca = ListChunked::from_chunks("", vec![Box::new(arr)]);
+            let mut ca = unsafe { ListChunked::from_chunks("", vec![Box::new(arr)]) };
 
-            if self.fast_explode {
+            if slf.fast_explode {
                 ca.set_fast_explode();
             }
 
@@ -631,10 +628,12 @@ impl ListBuilderTrait for AnonymousOwnedListBuilder {
 
     #[inline]
     fn append_null(&mut self) {
+        self.fast_explode = false;
         self.builder.push_null()
     }
 
     fn finish(&mut self) -> ListChunked {
+        // don't use self from here on one
         let slf = std::mem::take(self);
         if slf.builder.is_empty() {
             // not really empty, there were empty null list added probably e.g. []
@@ -644,7 +643,8 @@ impl ListBuilderTrait for AnonymousOwnedListBuilder {
                 let array = new_null_array(dtype.clone(), real_length);
                 let dtype = ListArray::<i64>::default_datatype(dtype);
                 let array = ListArray::new(dtype, slf.builder.take_offsets().into(), array, None);
-                ListChunked::from_chunks(&slf.name, vec![Box::new(array)])
+                // safety: same type
+                unsafe { ListChunked::from_chunks(&slf.name, vec![Box::new(array)]) }
             } else {
                 ListChunked::full_null_with_dtype(
                     &slf.name,
@@ -656,9 +656,10 @@ impl ListBuilderTrait for AnonymousOwnedListBuilder {
             let inner_dtype = slf.inner_dtype.map(|dt| dt.to_physical().to_arrow());
             let arr = slf.builder.finish(inner_dtype.as_ref()).unwrap();
             let dtype = DataType::from(arr.data_type());
-            let mut ca = ListChunked::from_chunks("", vec![Box::new(arr)]);
+            // safety: same type
+            let mut ca = unsafe { ListChunked::from_chunks("", vec![Box::new(arr)]) };
 
-            if self.fast_explode {
+            if slf.fast_explode {
                 ca.set_fast_explode();
             }
 
