@@ -121,7 +121,7 @@ where
         self.cast_impl(data_type, true)
     }
 
-    fn cast_unchecked(&self, data_type: &DataType) -> PolarsResult<Series> {
+    unsafe fn cast_unchecked(&self, data_type: &DataType) -> PolarsResult<Series> {
         match data_type {
             #[cfg(feature = "dtype-categorical")]
             DataType::Categorical(Some(rev_map)) => {
@@ -164,12 +164,11 @@ impl ChunkCast for Utf8Chunked {
         }
     }
 
-    fn cast_unchecked(&self, data_type: &DataType) -> PolarsResult<Series> {
+    unsafe fn cast_unchecked(&self, data_type: &DataType) -> PolarsResult<Series> {
         self.cast(data_type)
     }
 }
 
-#[cfg(feature = "dtype-binary")]
 unsafe fn binary_to_utf8_unchecked(from: &BinaryArray<i64>) -> Utf8Array<i64> {
     let values = from.values().clone();
     let offsets = from.offsets().clone();
@@ -182,7 +181,33 @@ unsafe fn binary_to_utf8_unchecked(from: &BinaryArray<i64>) -> Utf8Array<i64> {
     .unwrap()
 }
 
-#[cfg(feature = "dtype-binary")]
+impl BinaryChunked {
+    /// # Safety
+    /// Utf8 is nto validated
+    pub unsafe fn to_utf8(&self) -> Utf8Chunked {
+        let chunks = self
+            .downcast_iter()
+            .map(|arr| Box::new(binary_to_utf8_unchecked(arr)) as ArrayRef)
+            .collect();
+        Utf8Chunked::from_chunks(self.name(), chunks)
+    }
+}
+
+impl Utf8Chunked {
+    pub fn as_binary(&self) -> BinaryChunked {
+        let chunks = self
+            .downcast_iter()
+            .map(|arr| {
+                Box::new(arrow::compute::cast::utf8_to_binary(
+                    arr,
+                    ArrowDataType::LargeBinary,
+                )) as ArrayRef
+            })
+            .collect();
+        unsafe { BinaryChunked::from_chunks(self.name(), chunks) }
+    }
+}
+
 impl ChunkCast for BinaryChunked {
     fn cast(&self, data_type: &DataType) -> PolarsResult<Series> {
         match data_type {
@@ -192,15 +217,9 @@ impl ChunkCast for BinaryChunked {
         }
     }
 
-    fn cast_unchecked(&self, data_type: &DataType) -> PolarsResult<Series> {
+    unsafe fn cast_unchecked(&self, data_type: &DataType) -> PolarsResult<Series> {
         match data_type {
-            DataType::Utf8 => unsafe {
-                let chunks = self
-                    .downcast_iter()
-                    .map(|arr| Box::new(binary_to_utf8_unchecked(arr)) as ArrayRef)
-                    .collect();
-                Ok(Utf8Chunked::from_chunks(self.name(), chunks).into_series())
-            },
+            DataType::Utf8 => unsafe { Ok(self.to_utf8().into_series()) },
             _ => self.cast(data_type),
         }
     }
@@ -230,7 +249,7 @@ impl ChunkCast for BooleanChunked {
         }
     }
 
-    fn cast_unchecked(&self, data_type: &DataType) -> PolarsResult<Series> {
+    unsafe fn cast_unchecked(&self, data_type: &DataType) -> PolarsResult<Series> {
         self.cast(data_type)
     }
 }
@@ -290,7 +309,7 @@ impl ChunkCast for ListChunked {
         }
     }
 
-    fn cast_unchecked(&self, data_type: &DataType) -> PolarsResult<Series> {
+    unsafe fn cast_unchecked(&self, data_type: &DataType) -> PolarsResult<Series> {
         self.cast(data_type)
     }
 }
