@@ -1,4 +1,4 @@
-use num::{Float, NumCast};
+use num_traits::{Float, NumCast};
 use rand::distributions::Bernoulli;
 use rand::prelude::*;
 use rand_distr::{Distribution, Normal, Standard, StandardNormal, Uniform};
@@ -70,7 +70,7 @@ impl Series {
             ));
         }
         if n == 0 {
-            return Ok(self.slice(0, 0));
+            return Ok(self.clear());
         }
         let len = self.len();
 
