@@ -6,7 +6,7 @@ use std::fmt::Debug;
 use std::ops::{self, Add, Div, Mul, Sub};
 
 pub use borrowed::*;
-use num::{Num, NumCast};
+use num_traits::{Num, NumCast};
 
 use crate::prelude::*;
 use crate::utils::{get_time_units, try_get_supertype};
