@@ -1,6 +1,5 @@
 mod batched;
 
-use std::borrow::Cow;
 use std::fmt;
 use std::ops::Deref;
 use std::sync::atomic::{AtomicUsize, Ordering};
@@ -77,7 +76,7 @@ pub(crate) fn cast_columns(
 pub(crate) struct CoreReader<'a> {
     reader_bytes: Option<ReaderBytes<'a>>,
     /// Explicit schema for the CSV file
-    schema: Cow<'a, Schema>,
+    schema: SchemaRef,
     /// Optional projection for which columns to load (zero-based column indices)
     projection: Option<Vec<usize>>,
     /// Current line number, used in error reporting
@@ -173,11 +172,11 @@ impl<'a> CoreReader<'a> {
         delimiter: Option<u8>,
         has_header: bool,
         ignore_errors: bool,
-        schema: Option<&'a Schema>,
+        schema: Option<SchemaRef>,
         columns: Option<Vec<String>>,
         encoding: CsvEncoding,
         n_threads: Option<usize>,
-        schema_overwrite: Option<&'a Schema>,
+        schema_overwrite: Option<SchemaRef>,
         dtype_overwrite: Option<&'a [DataType]>,
         sample_size: usize,
         chunk_size: usize,
@@ -205,7 +204,7 @@ impl<'a> CoreReader<'a> {
         let delimiter = delimiter.unwrap_or(b',');
 
         let mut schema = match schema {
-            Some(schema) => Cow::Borrowed(schema),
+            Some(schema) => schema,
             None => {
                 {
                     // We keep track of the inferred schema bool
@@ -223,7 +222,7 @@ impl<'a> CoreReader<'a> {
                         delimiter,
                         max_records,
                         has_header,
-                        schema_overwrite,
+                        schema_overwrite.as_deref(),
                         &mut skip_rows,
                         skip_rows_after_header,
                         comment_char,
@@ -232,16 +231,15 @@ impl<'a> CoreReader<'a> {
                         null_values.as_ref(),
                         try_parse_dates,
                     )?;
-                    Cow::Owned(inferred_schema)
+                    Arc::new(inferred_schema)
                 }
             }
         };
         if let Some(dtypes) = dtype_overwrite {
-            let mut s = schema.into_owned();
+            let s = Arc::make_mut(&mut schema);
             for (index, dt) in dtypes.iter().enumerate() {
                 s.coerce_by_index(index, dt.clone()).unwrap();
             }
-            schema = Cow::Owned(s);
         }
 
         // create a null value for every column
@@ -343,17 +341,14 @@ impl<'a> CoreReader<'a> {
         Ok((bytes, starting_point_offset))
     }
 
-    #[allow(clippy::type_complexity)]
-    fn determine_file_chunks_and_statistics(
+    /// Estimates number of rows and optionally ensure we don't read more than `n_rows`
+    /// by slicing `bytes` to the upper bound.
+    fn estimate_rows_and_set_upper_bound<'b>(
         &self,
-        n_threads: &mut usize,
-        bytes: &'a [u8],
+        mut bytes: &'b [u8],
         logging: bool,
-        streaming: bool,
-    ) -> PolarsResult<(Vec<(usize, usize)>, usize, usize, Option<usize>, &'a [u8])> {
-        // Make the variable mutable so that we can reassign the sliced file to this variable.
-        let (mut bytes, starting_point_offset) = self.find_starting_point(bytes, self.eol_char)?;
-
+        set_upper_bound: bool,
+    ) -> (&'b [u8], usize) {
         // initial row guess. We use the line statistic to guess the number of rows to allocate
         let mut total_rows = 128;
 
@@ -390,7 +385,9 @@ impl<'a> CoreReader<'a> {
                         self.quote_char,
                         self.eol_char,
                     ) {
-                        bytes = &bytes[..n_bytes + pos]
+                        if set_upper_bound {
+                            bytes = &bytes[..n_bytes + pos]
+                        }
                     }
                 }
             }
@@ -398,6 +395,20 @@ impl<'a> CoreReader<'a> {
                 eprintln!("initial row estimate: {total_rows}")
             }
         }
+        (bytes, total_rows)
+    }
+
+    #[allow(clippy::type_complexity)]
+    fn determine_file_chunks_and_statistics(
+        &self,
+        n_threads: &mut usize,
+        bytes: &'a [u8],
+        logging: bool,
+    ) -> PolarsResult<(Vec<(usize, usize)>, usize, usize, Option<usize>, &'a [u8])> {
+        // Make the variable mutable so that we can reassign the sliced file to this variable.
+        let (bytes, starting_point_offset) = self.find_starting_point(bytes, self.eol_char)?;
+
+        let (bytes, total_rows) = self.estimate_rows_and_set_upper_bound(bytes, logging, true);
         if total_rows == 128 {
             *n_threads = 1;
 
@@ -407,9 +418,7 @@ impl<'a> CoreReader<'a> {
         }
 
         let chunk_size = std::cmp::min(self.chunk_size, total_rows);
-        let n_chunks = total_rows / chunk_size;
-
-        let n_file_chunks = if streaming { n_chunks } else { *n_threads };
+        let n_file_chunks = *n_threads;
 
         // split the file by the nearest new line characters such that every thread processes
         // approximately the same number of rows.
@@ -432,7 +441,6 @@ impl<'a> CoreReader<'a> {
 
         Ok((chunks, chunk_size, total_rows, starting_point_offset, bytes))
     }
-
     fn get_projection(&mut self) -> Vec<usize> {
         // we also need to sort the projection to have predictable output.
         // the `parse_lines` function expects this.
@@ -445,26 +453,33 @@ impl<'a> CoreReader<'a> {
             .unwrap_or_else(|| (0..self.schema.len()).collect())
     }
 
-    fn get_string_columns(&self, projection: &[usize]) -> PolarsResult<Vec<&str>> {
+    fn get_string_columns(&self, projection: &[usize]) -> PolarsResult<StringColumns> {
         // keep track of the maximum capacity that needs to be allocated for the utf8-builder
         // Per string column we keep a statistic of the maximum length of string bytes per chunk
         // We must the names, not the indexes, (the indexes are incorrect due to projection
         // pushdown)
-        let mut str_columns = Vec::with_capacity(projection.len());
+
+        let mut new_projection = Vec::with_capacity(projection.len());
+
         for i in projection {
-            let (name, dtype) = self.schema.get_index(*i).ok_or_else(||
+            let (_, dtype) = self.schema.get_index(*i).ok_or_else(||
                 PolarsError::ComputeError(
                     format!("the given projection index: {} is out of bounds for csv schema with {} columns", i, self.schema.len()).into())
             )?;
 
             if dtype == &DataType::Utf8 {
-                str_columns.push(name.as_str())
+                new_projection.push(*i)
             }
         }
-        Ok(str_columns)
+
+        Ok(StringColumns::new(self.schema.clone(), new_projection))
     }
 
-    fn init_string_size_stats(&self, str_columns: &[&str], capacity: usize) -> Vec<RunningSize> {
+    fn init_string_size_stats(
+        &self,
+        str_columns: &StringColumns,
+        capacity: usize,
+    ) -> Vec<RunningSize> {
         // assume 10 chars per str
         // this is not updated in low memory mode
         let init_str_bytes = capacity * 10;
@@ -482,7 +497,7 @@ impl<'a> CoreReader<'a> {
     ) -> PolarsResult<DataFrame> {
         let logging = verbose();
         let (file_chunks, chunk_size, total_rows, starting_point_offset, bytes) =
-            self.determine_file_chunks_and_statistics(&mut n_threads, bytes, logging, false)?;
+            self.determine_file_chunks_and_statistics(&mut n_threads, bytes, logging)?;
         let projection = self.get_projection();
         let str_columns = self.get_string_columns(&projection)?;
 
@@ -690,7 +705,7 @@ impl<'a> CoreReader<'a> {
 
 fn update_string_stats(
     str_capacities: &[RunningSize],
-    str_columns: &[&str],
+    str_columns: &StringColumns,
     local_df: &DataFrame,
 ) -> PolarsResult<()> {
     // update the running str bytes statistics
@@ -769,3 +784,27 @@ fn read_chunk(
             .collect::<PolarsResult<_>>()?,
     ))
 }
+
+/// List of strings, which are stored inside of a [Schema].
+///
+/// Conceptually it is `Vec<&str>` with `&str` tied to the lifetime of
+/// the [Schema].
+struct StringColumns {
+    schema: SchemaRef,
+    fields: Vec<usize>,
+}
+
+impl StringColumns {
+    /// New [StringColumns], where the list `fields` has indices
+    /// of fields in the `schema`.
+    fn new(schema: SchemaRef, fields: Vec<usize>) -> Self {
+        Self { schema, fields }
+    }
+
+    fn iter(&self) -> impl Iterator<Item = &str> {
+        self.fields.iter().map(|schema_i| {
+            let (name, _) = self.schema.get_index(*schema_i).unwrap();
+            name.as_str()
+        })
+    }
+}
