@@ -83,8 +83,7 @@ pub fn get_reader_bytes<R: Read + MmapBytesReader + ?Sized>(
 }
 
 static FLOAT_RE: Lazy<Regex> = Lazy::new(|| {
-    Regex::new(r"^(\s*-?((\d*\.\d+)[eE]?[-\+]?\d*)|[-+]?inf|[-+]?NaN|[-+]?\d+[eE][-+]\d+)$")
-        .unwrap()
+    Regex::new(r"^\s*[-+]?((\d*\.\d+)([eE][-+]?\d+)?|inf|NaN|(\d+)[eE][-+]?\d+|\d+\.)$").unwrap()
 });
 
 static INTEGER_RE: Lazy<Regex> = Lazy::new(|| Regex::new(r"^\s*-?(\d+)$").unwrap());
@@ -97,11 +96,11 @@ static BOOLEAN_RE: Lazy<Regex> = Lazy::new(|| {
 });
 
 /// Infer the data type of a record
-fn infer_field_schema(string: &str, parse_dates: bool) -> DataType {
+fn infer_field_schema(string: &str, try_parse_dates: bool) -> DataType {
     // when quoting is enabled in the reader, these quotes aren't escaped, we default to
     // Utf8 for them
     if string.starts_with('"') {
-        if parse_dates {
+        if try_parse_dates {
             #[cfg(feature = "polars-time")]
             {
                 match date_infer::infer_pattern_single(&string[1..string.len() - 1]) {
@@ -127,7 +126,7 @@ fn infer_field_schema(string: &str, parse_dates: bool) -> DataType {
         DataType::Float64
     } else if INTEGER_RE.is_match(string) {
         DataType::Int64
-    } else if parse_dates {
+    } else if try_parse_dates {
         #[cfg(feature = "polars-time")]
         {
             match date_infer::infer_pattern_single(string) {
@@ -185,7 +184,7 @@ pub fn infer_file_schema(
     quote_char: Option<u8>,
     eol_char: u8,
     null_values: Option<&NullValues>,
-    parse_dates: bool,
+    try_parse_dates: bool,
 ) -> PolarsResult<(Schema, usize, usize)> {
     // keep track so that we can determine the amount of bytes read
     let start_ptr = reader_bytes.as_ptr() as usize;
@@ -295,7 +294,7 @@ pub fn infer_file_schema(
             quote_char,
             eol_char,
             null_values,
-            parse_dates,
+            try_parse_dates,
         );
     } else {
         return Err(PolarsError::NoData("empty csv".into()));
@@ -358,16 +357,16 @@ pub fn infer_file_schema(
                     let s = parse_bytes_with_encoding(slice_escaped, encoding)?;
                     match &null_values {
                         None => {
-                            column_types[i].insert(infer_field_schema(&s, parse_dates));
+                            column_types[i].insert(infer_field_schema(&s, try_parse_dates));
                         }
                         Some(NullValues::AllColumns(names)) => {
                             if !names.iter().any(|nv| nv == s.as_ref()) {
-                                column_types[i].insert(infer_field_schema(&s, parse_dates));
+                                column_types[i].insert(infer_field_schema(&s, try_parse_dates));
                             }
                         }
                         Some(NullValues::AllColumnsSingle(name)) => {
                             if s.as_ref() != name {
-                                column_types[i].insert(infer_field_schema(&s, parse_dates));
+                                column_types[i].insert(infer_field_schema(&s, try_parse_dates));
                             }
                         }
                         Some(NullValues::Named(names)) => {
@@ -376,10 +375,10 @@ pub fn infer_file_schema(
 
                             if let Some(null_name) = null_name {
                                 if null_name.1 != s.as_ref() {
-                                    column_types[i].insert(infer_field_schema(&s, parse_dates));
+                                    column_types[i].insert(infer_field_schema(&s, try_parse_dates));
                                 }
                             } else {
-                                column_types[i].insert(infer_field_schema(&s, parse_dates));
+                                column_types[i].insert(infer_field_schema(&s, try_parse_dates));
                             }
                         }
                     }
@@ -398,6 +397,15 @@ pub fn infer_file_schema(
                 fields.push(Field::new(name, dtype.clone()));
                 continue;
             }
+
+            // column might have been renamed
+            // execute only if schema is complete
+            if schema_overwrite.len() == header_length {
+                if let Some((name, dtype)) = schema_overwrite.get_index(i) {
+                    fields.push(Field::new(name, dtype.clone()));
+                    continue;
+                }
+            }
         }
 
         // determine data type based on possible types
@@ -456,7 +464,7 @@ pub fn infer_file_schema(
             quote_char,
             eol_char,
             null_values,
-            parse_dates,
+            try_parse_dates,
         );
     }
 
@@ -516,9 +524,8 @@ fn decompress_impl<R: Read>(
                         break;
                     }
                     // now that we have enough, we compute the number of fields (also takes embedding into account)
-                    expected_fields = SplitFields::new(&out, delimiter, quote_char, eol_char)
-                        .into_iter()
-                        .count();
+                    expected_fields =
+                        SplitFields::new(&out, delimiter, quote_char, eol_char).count();
                     break;
                 }
             }
