@@ -5,6 +5,7 @@ use polars_io::csv::utils::{get_reader_bytes, infer_file_schema};
 use polars_io::csv::{CsvEncoding, NullValues};
 use polars_io::RowCount;
 
+use crate::frame::LazyFileListReader;
 use crate::prelude::*;
 
 #[derive(Clone)]
@@ -30,7 +31,7 @@ pub struct LazyCsvReader<'a> {
     skip_rows_after_header: usize,
     encoding: CsvEncoding,
     row_count: Option<RowCount>,
-    parse_dates: bool,
+    try_parse_dates: bool,
 }
 
 #[cfg(feature = "csv-file")]
@@ -57,7 +58,7 @@ impl<'a> LazyCsvReader<'a> {
             skip_rows_after_header: 0,
             encoding: CsvEncoding::Utf8,
             row_count: None,
-            parse_dates: false,
+            try_parse_dates: false,
         }
     }
 
@@ -183,13 +184,6 @@ impl<'a> LazyCsvReader<'a> {
         self
     }
 
-    /// Rechunk the memory to contiguous chunks when parsing is done.
-    #[must_use]
-    pub fn with_rechunk(mut self, toggle: bool) -> Self {
-        self.rechunk = toggle;
-        self
-    }
-
     /// Set  [`CsvEncoding`]
     #[must_use]
     pub fn with_encoding(mut self, enc: CsvEncoding) -> Self {
@@ -199,8 +193,8 @@ impl<'a> LazyCsvReader<'a> {
 
     /// Automatically try to parse dates/ datetimes and time. If parsing fails, columns remain of dtype `[DataType::Utf8]`.
     #[cfg(feature = "temporal")]
-    pub fn with_parse_dates(mut self, toggle: bool) -> Self {
-        self.parse_dates = toggle;
+    pub fn with_try_parse_dates(mut self, toggle: bool) -> Self {
+        self.try_parse_dates = toggle;
         self
     }
 
@@ -212,15 +206,11 @@ impl<'a> LazyCsvReader<'a> {
         F: Fn(Schema) -> PolarsResult<Schema>,
     {
         let path;
-        let path_str = self.path.to_string_lossy();
-
-        let mut file = if path_str.contains('*') {
-            let glob_err = || PolarsError::ComputeError("invalid glob pattern given".into());
-            let mut paths = glob::glob(&path_str).map_err(|_| glob_err())?;
 
+        let mut file = if let Some(mut paths) = self.glob()? {
             match paths.next() {
                 Some(globresult) => {
-                    path = globresult.map_err(|_| glob_err())?;
+                    path = globresult?;
                 }
                 None => {
                     return Err(PolarsError::ComputeError(
@@ -248,7 +238,7 @@ impl<'a> LazyCsvReader<'a> {
             self.quote_char,
             self.eol_char,
             None,
-            self.parse_dates,
+            self.try_parse_dates,
         )?;
         let mut schema = f(schema)?;
 
@@ -261,8 +251,10 @@ impl<'a> LazyCsvReader<'a> {
 
         Ok(self.with_schema(Arc::new(schema)))
     }
+}
 
-    pub fn finish_impl(self) -> PolarsResult<LazyFrame> {
+impl LazyFileListReader for LazyCsvReader<'_> {
+    fn finish_no_glob(self) -> PolarsResult<LazyFrame> {
         let mut lf: LazyFrame = LogicalPlanBuilder::scan_csv(
             self.path,
             self.delimiter,
@@ -283,7 +275,7 @@ impl<'a> LazyCsvReader<'a> {
             self.skip_rows_after_header,
             self.encoding,
             self.row_count,
-            self.parse_dates,
+            self.try_parse_dates,
         )?
         .build()
         .into();
@@ -291,27 +283,39 @@ impl<'a> LazyCsvReader<'a> {
         Ok(lf)
     }
 
-    pub fn finish(self) -> PolarsResult<LazyFrame> {
-        let path_str = self.path.to_string_lossy();
-        if path_str.contains('*') {
-            let paths = glob::glob(&path_str)
-                .map_err(|_| PolarsError::ComputeError("invalid glob pattern given".into()))?;
+    fn path(&self) -> &Path {
+        &self.path
+    }
 
-            let lfs = paths
-                .map(|r| {
-                    let path = r.map_err(|e| PolarsError::ComputeError(format!("{e}").into()))?;
-                    let mut builder = self.clone();
-                    builder.path = path;
-                    // do no rechunk yet.
-                    builder.rechunk = false;
-                    builder.finish_impl()
-                })
-                .collect::<PolarsResult<Vec<_>>>()?;
-            // set to false, as the csv parser has full thread utilization
-            concat_impl(&lfs, self.rechunk, false, true)
-                .map_err(|_| PolarsError::ComputeError("no matching files found".into()))
-        } else {
-            self.finish_impl()
-        }
+    fn with_path(mut self, path: PathBuf) -> Self {
+        self.path = path;
+        self
+    }
+
+    fn rechunk(&self) -> bool {
+        self.rechunk
+    }
+
+    /// Rechunk the memory to contiguous chunks when parsing is done.
+    #[must_use]
+    fn with_rechunk(mut self, toggle: bool) -> Self {
+        self.rechunk = toggle;
+        self
+    }
+
+    /// Try to stop parsing when `n` rows are parsed. During multithreaded parsing the upper bound `n` cannot
+    /// be guaranteed.
+    fn n_rows(&self) -> Option<usize> {
+        self.n_rows
+    }
+
+    /// Add a `row_count` column.
+    fn row_count(&self) -> Option<&RowCount> {
+        self.row_count.as_ref()
+    }
+
+    fn concat_impl(&self, lfs: Vec<LazyFrame>) -> PolarsResult<LazyFrame> {
+        // set to false, as the csv parser has full thread utilization
+        concat_impl(&lfs, self.rechunk(), false, true)
     }
 }
