@@ -154,7 +154,7 @@ impl BinaryExpr {
     fn apply_group_aware<'a>(
         &self,
         mut ac_l: AggregationContext<'a>,
-        mut ac_r: AggregationContext,
+        mut ac_r: AggregationContext<'a>,
     ) -> PolarsResult<AggregationContext<'a>> {
         if self.null_propagate(&mut ac_l, &ac_r) {
             return Ok(ac_l);
@@ -178,8 +178,22 @@ impl BinaryExpr {
             .collect::<PolarsResult<_>>()?;
         ca.rename(&name);
 
+        // try if we can reuse the groups
+        use AggState::*;
+        match (ac_l.agg_state(), ac_r.agg_state()) {
+            // no need to change update groups
+            (AggregatedList(_), _) => {}
+            // we can take the groups of the rhs
+            (_, AggregatedList(_)) if matches!(ac_r.update_groups, UpdateGroups::No) => {
+                ac_l.groups = ac_r.groups
+            }
+            // we must update the groups
+            _ => {
+                ac_l.with_update_groups(UpdateGroups::WithSeriesLen);
+            }
+        }
+
         ac_l.with_series(ca.into_series(), true, Some(&self.expr))?;
-        ac_l.with_update_groups(UpdateGroups::WithSeriesLen);
         Ok(ac_l)
     }
 }
@@ -282,8 +296,31 @@ mod stats {
 
     use super::*;
 
+    fn apply_operator_stats_eq(min_max: &Series, literal: &Series) -> bool {
+        // literal is greater than max, don't need to read
+        if ChunkCompare::<&Series>::gt(literal, min_max)
+            .ok()
+            .map(|s| s.all())
+            == Some(true)
+        {
+            return false;
+        }
+
+        // literal is smaller than min, don't need to read
+        if ChunkCompare::<&Series>::lt(literal, min_max)
+            .ok()
+            .map(|s| s.all())
+            == Some(true)
+        {
+            return false;
+        }
+
+        true
+    }
+
     fn apply_operator_stats_rhs_lit(min_max: &Series, literal: &Series, op: Operator) -> bool {
         match op {
+            Operator::Eq => apply_operator_stats_eq(min_max, literal),
             // col > lit
             // e.g.
             // [min,
@@ -335,6 +372,7 @@ mod stats {
 
     fn apply_operator_stats_lhs_lit(literal: &Series, min_max: &Series, op: Operator) -> bool {
         match op {
+            Operator::Eq => apply_operator_stats_eq(min_max, literal),
             Operator::Gt => {
                 // literal is bigger than max value
                 // selection needs all rows
@@ -380,7 +418,7 @@ mod stats {
             if !self.expr.into_iter().all(|e| match e {
                 BinaryExpr { op, .. } => !matches!(
                     op,
-                    Multiply | Divide | TrueDivide | FloorDivide | Modulus | Eq | NotEq
+                    Multiply | Divide | TrueDivide | FloorDivide | Modulus | NotEq
                 ),
                 Column(_) | Literal(_) | Alias(_, _) => true,
                 _ => false,
