@@ -9,7 +9,7 @@ use polars_arrow::export::arrow::{self};
 use polars_arrow::kernels::string::*;
 #[cfg(feature = "string_from_radix")]
 use polars_core::export::num::Num;
-use polars_core::export::regex::{escape, NoExpand, Regex};
+use polars_core::export::regex::{escape, Regex};
 
 use super::*;
 #[cfg(feature = "binary_encoding")]
@@ -23,15 +23,13 @@ fn f_regex_extract<'a>(reg: &Regex, input: &'a str, group_index: usize) -> Optio
 pub trait Utf8NameSpaceImpl: AsUtf8 {
     #[cfg(not(feature = "binary_encoding"))]
     fn hex_decode(&self) -> PolarsResult<Utf8Chunked> {
-        panic!("activate 'dtype-binary' feature")
+        panic!("activate 'binary_encoding' feature")
     }
 
     #[cfg(feature = "binary_encoding")]
     fn hex_decode(&self, strict: bool) -> PolarsResult<BinaryChunked> {
         let ca = self.as_utf8();
-        ca.cast_unchecked(&DataType::Binary)?
-            .binary()?
-            .hex_decode(strict)
+        ca.as_binary().hex_decode(strict)
     }
 
     #[must_use]
@@ -43,15 +41,13 @@ pub trait Utf8NameSpaceImpl: AsUtf8 {
 
     #[cfg(not(feature = "binary_encoding"))]
     fn base64_decode(&self) -> PolarsResult<Utf8Chunked> {
-        panic!("activate 'dtype-binary' feature")
+        panic!("activate 'binary_encoding' feature")
     }
 
     #[cfg(feature = "binary_encoding")]
     fn base64_decode(&self, strict: bool) -> PolarsResult<BinaryChunked> {
         let ca = self.as_utf8();
-        ca.cast_unchecked(&DataType::Binary)?
-            .binary()?
-            .base64_decode(strict)
+        ca.as_binary().base64_decode(strict)
     }
 
     #[must_use]
@@ -63,11 +59,40 @@ pub trait Utf8NameSpaceImpl: AsUtf8 {
 
     #[cfg(feature = "string_from_radix")]
     // Parse a string number with base _radix_ into a decimal (i32)
-    fn parse_int(&self, radix: Option<u32>) -> Int32Chunked {
+    fn parse_int(&self, radix: u32, strict: bool) -> PolarsResult<Int32Chunked> {
+        use polars_arrow::utils::CustomIterTools;
         let ca = self.as_utf8();
+        let f = |opt_s: Option<&str>| -> Option<i32> {
+            opt_s.and_then(|s| <i32 as Num>::from_str_radix(s, radix).ok())
+        };
+        let out: Int32Chunked = ca.into_iter().map(f).collect_trusted();
 
-        let f = |s: &str| <i32 as Num>::from_str_radix(s, radix.unwrap_or(2)).unwrap();
-        ca.apply_cast_numeric(f)
+        if strict && ca.null_count() != out.null_count() {
+            let failure_mask = !ca.is_null() & out.is_null();
+            let all_failures = ca.filter(&failure_mask)?;
+            let n_failures = all_failures.len();
+            let some_failures = all_failures.unique()?.slice(0, 10).sort(false);
+            let some_error_msg = some_failures
+                .get(0)
+                .and_then(|s| <i32 as Num>::from_str_radix(s, radix).err())
+                .map_or_else(
+                    || unreachable!("Could unexpectedly not recreate ParseIntError"),
+                    |err| format!("{}", err),
+                );
+
+            Err(PolarsError::ComputeError(
+                format!(
+                    "Strict integer parsing failed for {} value(s): {} \
+                    ParseIntError msg for first shown value: '{}'. Consider non strict parsing.",
+                    n_failures,
+                    some_failures.into_series().fmt_list(),
+                    some_error_msg
+                )
+                .into(),
+            ))?
+        };
+
+        Ok(out)
     }
 
     /// Get the length of the string values as number of chars.
@@ -218,13 +243,34 @@ pub trait Utf8NameSpaceImpl: AsUtf8 {
 
     /// Replace the leftmost literal (sub)string with another string
     fn replace_literal<'a>(&'a self, pat: &str, val: &str) -> PolarsResult<Utf8Chunked> {
-        // note: benchmarking shows that using the regex engine for literal
-        // replacement is faster than str::replacen in almost all cases.
-        // ref: https://github.com/pola-rs/polars/pull/6777
-        let reg = Regex::new(escape(pat).as_str())?;
-        let f = |s: &'a str| reg.replace(s, NoExpand(val));
+        // amortize allocation
+        let mut buf = String::new();
+
+        let f = move |s: &'a str| {
+            buf.clear();
+            let mut changed = false;
+
+            // See: str.replacen
+            let mut last_end = 0;
+            if let Some((start, part)) = s.match_indices(pat).next() {
+                changed = true;
+                buf.push_str(unsafe { s.get_unchecked(last_end..start) });
+                buf.push_str(val);
+                last_end = start + part.len();
+            }
+            buf.push_str(unsafe { s.get_unchecked(last_end..s.len()) });
+
+            if changed {
+                // extend lifetime
+                // lifetime is bound to 'a
+                let slice = buf.as_str();
+                unsafe { std::mem::transmute::<&str, &'a str>(slice) }
+            } else {
+                s
+            }
+        };
         let ca = self.as_utf8();
-        Ok(ca.apply(f))
+        Ok(ca.apply_mut(f))
     }
 
     /// Replace all regex-matched (sub)strings with another string
@@ -235,10 +281,36 @@ pub trait Utf8NameSpaceImpl: AsUtf8 {
     }
 
     /// Replace all matching literal (sub)strings with another string
-    fn replace_literal_all(&self, pat: &str, val: &str) -> PolarsResult<Utf8Chunked> {
+    fn replace_literal_all<'a>(&'a self, pat: &str, val: &str) -> PolarsResult<Utf8Chunked> {
+        // amortize allocation
+        let mut buf = String::new();
+
+        let f = move |s: &'a str| {
+            buf.clear();
+            let mut changed = false;
+
+            // See: str.replace
+            let mut last_end = 0;
+            for (start, part) in s.match_indices(pat) {
+                changed = true;
+                buf.push_str(unsafe { s.get_unchecked(last_end..start) });
+                buf.push_str(val);
+                last_end = start + part.len();
+            }
+            buf.push_str(unsafe { s.get_unchecked(last_end..s.len()) });
+
+            if changed {
+                // extend lifetime
+                // lifetime is bound to 'a
+                let slice = buf.as_str();
+                unsafe { std::mem::transmute::<&str, &'a str>(slice) }
+            } else {
+                s
+            }
+        };
+
         let ca = self.as_utf8();
-        let reg = Regex::new(escape(pat).as_str())?;
-        Ok(ca.apply(|s| reg.replace_all(s, NoExpand(val))))
+        Ok(ca.apply_mut(f))
     }
 
     /// Extract the nth capture group from pattern
