@@ -25,7 +25,7 @@ pub enum TemporalFunction {
     Truncate(String, String),
     Round(String, String),
     #[cfg(feature = "timezones")]
-    CastTimezone(TimeZone),
+    CastTimezone(Option<TimeZone>),
     #[cfg(feature = "timezones")]
     TzLocalize(TimeZone),
     DateRange {
@@ -59,7 +59,7 @@ impl Display for TemporalFunction {
             Truncate(..) => "truncate",
             Round(..) => "round",
             #[cfg(feature = "timezones")]
-            CastTimezone(_) => "cast_timezone",
+            CastTimezone(_) => "replace_timezone",
             #[cfg(feature = "timezones")]
             TzLocalize(_) => "tz_localize",
             DateRange { .. } => return write!(f, "date_range"),
@@ -137,23 +137,21 @@ pub(super) fn round(s: &Series, every: &str, offset: &str) -> PolarsResult<Serie
     }
 }
 #[cfg(feature = "timezones")]
-pub(super) fn cast_timezone(s: &Series, tz: &str) -> PolarsResult<Series> {
+pub(super) fn replace_timezone(s: &Series, time_zone: Option<&str>) -> PolarsResult<Series> {
     let ca = s.datetime()?;
-    ca.cast_time_zone(tz).map(|ca| ca.into_series())
+    ca.replace_time_zone(time_zone).map(|ca| ca.into_series())
 }
 
 #[cfg(feature = "timezones")]
+#[deprecated(note = "use replace_time_zone")]
 pub(super) fn tz_localize(s: &Series, tz: &str) -> PolarsResult<Series> {
     let ca = s.datetime()?.clone();
-    match ca.time_zone() {
-        Some(tz) if tz == "UTC" => {
-           Ok(ca.with_time_zone(Some("UTC".into())).into_series())
-        }
-        Some(tz) if !tz.is_empty() => {
-            Err(PolarsError::ComputeError("Cannot localize a tz-aware datetime. Consider using 'dt.with_time_zone' or 'dt.cast_time_zone'".into()))
+    match (ca.time_zone(), tz) {
+        (Some(old_tz), _) if !old_tz.is_empty() => {
+            Err(PolarsError::ComputeError("Cannot localize a tz-aware datetime. Consider using 'dt.convert_time_zone' or 'dt.replace_time_zone'".into()))
         },
-        _ => {
-            Ok(ca.with_time_zone(Some("UTC".to_string())).cast_time_zone(tz)?.into_series())
+        (_, _) => {
+            Ok(ca.replace_time_zone(Some(tz))?.into_series())
         }
     }
 }
@@ -192,7 +190,7 @@ pub(super) fn date_range_dispatch(
                     closed,
                     TimeUnit::Milliseconds,
                     tz.as_ref(),
-                )
+                )?
                 .cast(&DataType::Date)
             }
             DataType::Datetime(tu, _) => {
@@ -202,7 +200,7 @@ pub(super) fn date_range_dispatch(
                 let stop = stop.get(0).unwrap().extract::<i64>().unwrap();
 
                 Ok(
-                    date_range_impl(name, start, stop, every, closed, *tu, tz.as_ref())
+                    date_range_impl(name, start, stop, every, closed, *tu, tz.as_ref())?
                         .into_series(),
                 )
             }
@@ -239,7 +237,7 @@ pub(super) fn date_range_dispatch(
                     match (start, stop) {
                         (Some(start), Some(stop)) => {
                             let date_range =
-                                date_range_impl("", start, stop, every, closed, tu, tz);
+                                date_range_impl("", start, stop, every, closed, tu, tz)?;
                             let date_range = date_range.cast(&DataType::Date).unwrap();
                             let date_range = date_range.to_physical_repr();
                             let date_range = date_range.i32().unwrap();
@@ -262,7 +260,7 @@ pub(super) fn date_range_dispatch(
                     match (start, stop) {
                         (Some(start), Some(stop)) => {
                             let date_range =
-                                date_range_impl("", start, stop, every, closed, tu, tz);
+                                date_range_impl("", start, stop, every, closed, tu, tz)?;
                             builder.append_slice(date_range.cont_slice().unwrap())
                         }
                         _ => builder.append_null(),
