@@ -443,6 +443,10 @@ where
     out
 }
 
+fn is_literal_pat(pat: &str) -> bool {
+    pat.chars().all(|c| !c.is_ascii_punctuation())
+}
+
 #[cfg(feature = "regex")]
 fn replace_single<'a>(
     ca: &'a Utf8Chunked,
@@ -454,6 +458,7 @@ fn replace_single<'a>(
         (1, 1) => {
             let pat = get_pat(pat)?;
             let val = val.get(0).ok_or_else(|| PolarsError::ComputeError("value may not be 'null' in 'replace' expression".into()))?;
+            let literal = literal || is_literal_pat(pat);
 
             match literal {
                 true => ca.replace_literal(pat, val),
@@ -466,6 +471,7 @@ fn replace_single<'a>(
                 return Err(PolarsError::ComputeError(format!("The replacement value expression in 'str.replace' should be equal to the length of the string column.\
                 Got column length: {} and replacement value length: {}", ca.len(), len_val).into()))
             }
+            let literal = literal || is_literal_pat(&pat);
 
             if literal {
                 pat = escape(&pat)
@@ -499,6 +505,7 @@ fn replace_all<'a>(
         (1, 1) => {
             let pat = get_pat(pat)?;
             let val = val.get(0).ok_or_else(|| PolarsError::ComputeError("value may not be 'null' in 'replace' expression".into()))?;
+            let literal = literal || is_literal_pat(pat);
 
             match literal {
                 true => ca.replace_literal_all(pat, val),
@@ -511,6 +518,7 @@ fn replace_all<'a>(
                 return Err(PolarsError::ComputeError(format!("The replacement value expression in 'str.replace' should be equal to the length of the string column.\
                 Got column length: {} and replacement value length: {}", ca.len(), len_val).into()))
             }
+            let literal = literal || is_literal_pat(&pat);
 
             if literal {
                 pat = escape(&pat)
