@@ -204,10 +204,6 @@ impl LogicalPlan {
                 writeln!(f, "{:indent$}SORT BY {by_column:?}", "")?;
                 input._format(f, indent)
             }
-            Explode { input, columns, .. } => {
-                writeln!(f, "{:indent$}EXPLODE BY {columns:?}", "")?;
-                input._format(f, indent)
-            }
             Aggregate {
                 input, keys, aggs, ..
             } => {
@@ -439,7 +435,6 @@ impl Debug for LiteralValue {
             Null => write!(f, "null"),
             Boolean(b) => write!(f, "{b}"),
             Utf8(s) => write!(f, "{s}"),
-            #[cfg(feature = "dtype-binary")]
             Binary(_) => write!(f, "[binary value]"),
             #[cfg(feature = "dtype-u8")]
             UInt8(v) => write!(f, "{v}u8"),
