@@ -376,23 +376,6 @@ impl PredicatePushDown {
                 }
             }
 
-            Explode { input, columns, schema } => {
-                let condition = |name: Arc<str>| columns.iter().any(|s| s.as_str() == &*name);
-
-                // first columns that refer to the exploded columns should be done here
-                let mut local_predicates =
-                    transfer_to_local_by_name(expr_arena, &mut acc_predicates, condition);
-
-                // if any predicate is a pushdown boundary, thus influenced by order of predicates e.g.: sum(), over(), sort
-                // we do all here. #5950
-                if acc_predicates.values().chain(local_predicates.iter()).any(|node| predicate_is_pushdown_boundary(*node, expr_arena)) {
-                    local_predicates.extend(acc_predicates.drain().map(|(_name, node)| node))
-                }
-
-                self.pushdown_and_assign(input, acc_predicates, lp_arena, expr_arena)?;
-                let lp = Explode { input, columns, schema };
-                Ok(self.optional_apply_predicate(lp, local_predicates, lp_arena, expr_arena))
-            }
             Distinct {
                 input,
                 options
@@ -539,7 +522,26 @@ impl PredicatePushDown {
                             )?;
                             let lp = self.pushdown_and_continue(lp, acc_predicates, lp_arena, expr_arena, false)?;
                             Ok(self.optional_apply_predicate(lp, local_predicates, lp_arena, expr_arena))
-                        }, _ => {
+                        },
+                        FunctionNode::Explode {columns, ..} => {
+
+                            let condition = |name: Arc<str>| columns.iter().any(|s| s.as_ref() == &*name);
+
+                            // first columns that refer to the exploded columns should be done here
+                            let mut local_predicates =
+                                transfer_to_local_by_name(expr_arena, &mut acc_predicates, condition);
+
+                            // if any predicate is a pushdown boundary, thus influenced by order of predicates e.g.: sum(), over(), sort
+                            // we do all here. #5950
+                            if acc_predicates.values().chain(local_predicates.iter()).any(|node| predicate_is_pushdown_boundary(*node, expr_arena)) {
+                                local_predicates.extend(acc_predicates.drain().map(|(_name, node)| node))
+                            }
+
+                            let lp = self.pushdown_and_continue(lp, acc_predicates, lp_arena, expr_arena, false)?;
+                            Ok(self.optional_apply_predicate(lp, local_predicates, lp_arena, expr_arena))
+
+                        }
+                        _ => {
                             self.pushdown_and_continue(lp, acc_predicates, lp_arena, expr_arena, false)
                         }
                     }
