@@ -1,3 +1,4 @@
+mod functions;
 mod generic;
 mod groupby;
 mod hstack;
@@ -562,33 +563,6 @@ impl ProjectionPushDown {
                     args,
                 })
             }
-            Explode {
-                input,
-                columns,
-                schema,
-            } => {
-                columns.iter().for_each(|name| {
-                    add_str_to_accumulated(
-                        name,
-                        &mut acc_projections,
-                        &mut projected_names,
-                        expr_arena,
-                    )
-                });
-                self.pushdown_and_assign(
-                    input,
-                    acc_projections,
-                    projected_names,
-                    projections_seen,
-                    lp_arena,
-                    expr_arena,
-                )?;
-                Ok(Explode {
-                    input,
-                    columns,
-                    schema,
-                })
-            }
             Distinct { input, options } => {
                 // make sure that the set of unique columns is projected
                 if let Some(subset) = options.subset.as_ref() {
@@ -755,88 +729,16 @@ impl ProjectionPushDown {
             MapFunction {
                 input,
                 ref function,
-            } => {
-                let lp = MapFunction {
-                    input,
-                    function: function.clone(),
-                };
-
-                if let FunctionNode::Rename {
-                    existing,
-                    new,
-                    swapping,
-                } = function
-                {
-                    process_rename(
-                        &mut acc_projections,
-                        &mut projected_names,
-                        expr_arena,
-                        existing,
-                        new,
-                        *swapping,
-                    )?;
-                    self.pushdown_and_assign(
-                        input,
-                        acc_projections,
-                        projected_names,
-                        projections_seen,
-                        lp_arena,
-                        expr_arena,
-                    )?;
-                    return Ok(lp);
-                }
-
-                let MapFunction {ref function, ..} = lp else { unreachable!() };
-
-                if function.allow_projection_pd() && !acc_projections.is_empty() {
-                    let original_acc_projection_len = acc_projections.len();
-
-                    // add columns needed for the function.
-                    for name in function.additional_projection_pd_columns() {
-                        let node = expr_arena.add(AExpr::Column(name.clone()));
-                        add_expr_to_accumulated(
-                            node,
-                            &mut acc_projections,
-                            &mut projected_names,
-                            expr_arena,
-                        )
-                    }
-                    let expands_schema = matches!(function, FunctionNode::Unnest { .. });
-
-                    let local_projections = self.pushdown_and_assign_check_schema(
-                        input,
-                        acc_projections,
-                        projections_seen,
-                        lp_arena,
-                        expr_arena,
-                        expands_schema,
-                    )?;
-                    if local_projections.is_empty() {
-                        Ok(lp)
-                    } else {
-                        // if we would project, we would remove pushed down predicates
-                        if local_projections.len() < original_acc_projection_len {
-                            Ok(ALogicalPlanBuilder::from_lp(lp, expr_arena, lp_arena)
-                                .with_columns(local_projections)
-                                .build())
-                            // all projections are local
-                        } else {
-                            Ok(ALogicalPlanBuilder::from_lp(lp, expr_arena, lp_arena)
-                                .project(local_projections)
-                                .build())
-                        }
-                    }
-                } else {
-                    // restart projection pushdown
-                    self.no_pushdown_restart_opt(
-                        lp,
-                        acc_projections,
-                        projections_seen,
-                        lp_arena,
-                        expr_arena,
-                    )
-                }
-            }
+            } => functions::process_functions(
+                self,
+                input,
+                function,
+                acc_projections,
+                projected_names,
+                projections_seen,
+                lp_arena,
+                expr_arena,
+            ),
             lp @ Union { .. } => {
                 self.has_joins_or_unions = true;
                 process_generic(
