@@ -131,6 +131,7 @@ impl SlicePushDown {
                 output_schema,
                 predicate,
                 options,
+                cloud_options,
 
             },
                 // TODO! we currently skip slice pushdown if there is a predicate.
@@ -143,7 +144,8 @@ impl SlicePushDown {
                     file_info,
                     output_schema,
                     predicate,
-                    options
+                    options,
+                    cloud_options,
                 };
 
                 Ok(lp)
@@ -298,7 +300,7 @@ impl SlicePushDown {
             // other blocking nodes
             | m @ (DataFrameScan {..}, _)
             | m @ (Sort {..}, _)
-            | m @ (Explode {..}, _)
+            | m @ (MapFunction {function: FunctionNode::Explode {..}, ..}, _)
             | m @ (Melt {..}, _)
             | m @ (Cache {..}, _)
             | m @ (Distinct {..}, _)
