@@ -23,13 +23,13 @@ fn test_date_range() {
         TimeUnit::Nanoseconds,
     );
     let expected = [
-        NaiveDate::from_ymd(2022, 1, 1),
-        NaiveDate::from_ymd(2022, 2, 1),
-        NaiveDate::from_ymd(2022, 3, 1),
-        NaiveDate::from_ymd(2022, 4, 1),
+        NaiveDate::from_ymd_opt(2022, 1, 1).unwrap(),
+        NaiveDate::from_ymd_opt(2022, 2, 1).unwrap(),
+        NaiveDate::from_ymd_opt(2022, 3, 1).unwrap(),
+        NaiveDate::from_ymd_opt(2022, 4, 1).unwrap(),
     ]
     .iter()
-    .map(|d| d.and_hms(0, 0, 0).timestamp_nanos())
+    .map(|d| d.and_hms_opt(0, 0, 0).unwrap().timestamp_nanos())
     .collect::<Vec<_>>();
     assert_eq!(dates, expected);
 }
@@ -52,11 +52,11 @@ fn test_feb_date_range() {
         TimeUnit::Nanoseconds,
     );
     let expected = [
-        NaiveDate::from_ymd(2022, 2, 1),
-        NaiveDate::from_ymd(2022, 3, 1),
+        NaiveDate::from_ymd_opt(2022, 2, 1).unwrap(),
+        NaiveDate::from_ymd_opt(2022, 3, 1).unwrap(),
     ]
     .iter()
-    .map(|d| d.and_hms(0, 0, 0).timestamp_nanos())
+    .map(|d| d.and_hms_opt(0, 0, 0).unwrap().timestamp_nanos())
     .collect::<Vec<_>>();
     assert_eq!(dates, expected);
 }
@@ -77,14 +77,14 @@ fn take_groups_slice<'a>(groups: &'a GroupsSlice, idx: usize, ts: &'a [i64]) ->
 #[test]
 fn test_groups_large_interval() {
     let dates = &[
-        NaiveDate::from_ymd(2020, 1, 1),
-        NaiveDate::from_ymd(2020, 1, 11),
-        NaiveDate::from_ymd(2020, 1, 12),
-        NaiveDate::from_ymd(2020, 1, 13),
+        NaiveDate::from_ymd_opt(2020, 1, 1).unwrap(),
+        NaiveDate::from_ymd_opt(2020, 1, 11).unwrap(),
+        NaiveDate::from_ymd_opt(2020, 1, 12).unwrap(),
+        NaiveDate::from_ymd_opt(2020, 1, 13).unwrap(),
     ];
     let ts = dates
         .iter()
-        .map(|d| d.and_hms(0, 0, 0).timestamp_nanos())
+        .map(|d| d.and_hms_opt(0, 0, 0).unwrap().timestamp_nanos())
         .collect::<Vec<_>>();
 
     let dur = Duration::parse("2d");
@@ -96,6 +96,7 @@ fn test_groups_large_interval() {
         TimeUnit::Nanoseconds,
         false,
         false,
+        Default::default(),
     );
     assert_eq!(groups.len(), 4);
     assert_eq!(groups[0], [0, 1]);
@@ -109,6 +110,7 @@ fn test_groups_large_interval() {
         TimeUnit::Nanoseconds,
         false,
         false,
+        Default::default(),
     );
     assert_eq!(groups.len(), 3);
     assert_eq!(groups[2], [3, 1]);
@@ -119,6 +121,7 @@ fn test_groups_large_interval() {
         TimeUnit::Nanoseconds,
         false,
         false,
+        Default::default(),
     );
     assert_eq!(groups.len(), 2);
     assert_eq!(groups[1], [2, 2]);
@@ -126,8 +129,10 @@ fn test_groups_large_interval() {
 
 #[test]
 fn test_offset() {
-    let t = NaiveDate::from_ymd(2020, 1, 2)
-        .and_hms(0, 0, 0)
+    let t = NaiveDate::from_ymd_opt(2020, 1, 2)
+        .unwrap()
+        .and_hms_opt(0, 0, 0)
+        .unwrap()
         .timestamp_nanos();
     let w = Window::new(
         Duration::parse("5m"),
@@ -136,16 +141,24 @@ fn test_offset() {
     );
 
     let b = w.get_earliest_bounds_ns(t);
-    let start = NaiveDate::from_ymd(2020, 1, 1)
-        .and_hms(23, 58, 0)
+    let start = NaiveDate::from_ymd_opt(2020, 1, 1)
+        .unwrap()
+        .and_hms_opt(23, 58, 0)
+        .unwrap()
         .timestamp_nanos();
     assert_eq!(b.start, start);
 }
 
 #[test]
 fn test_boundaries() {
-    let start = NaiveDate::from_ymd(2021, 12, 16).and_hms(0, 0, 0);
-    let stop = NaiveDate::from_ymd(2021, 12, 16).and_hms(3, 0, 0);
+    let start = NaiveDate::from_ymd_opt(2021, 12, 16)
+        .unwrap()
+        .and_hms_opt(0, 0, 0)
+        .unwrap();
+    let stop = NaiveDate::from_ymd_opt(2021, 12, 16)
+        .unwrap()
+        .and_hms_opt(3, 0, 0)
+        .unwrap();
 
     let ts = date_range_vec(
         start.timestamp_nanos(),
@@ -176,6 +189,7 @@ fn test_boundaries() {
         TimeUnit::Nanoseconds,
         true,
         true,
+        Default::default(),
     );
 
     // 1st group
@@ -186,9 +200,18 @@ fn test_boundaries() {
     // 2021-12-16 00:30:00
     // 2021-12-16 01:00:00
     let g = take_groups_slice(&groups, 0, &ts);
-    let t0 = NaiveDate::from_ymd(2021, 12, 16).and_hms(0, 0, 0);
-    let t1 = NaiveDate::from_ymd(2021, 12, 16).and_hms(0, 30, 0);
-    let t2 = NaiveDate::from_ymd(2021, 12, 16).and_hms(1, 0, 0);
+    let t0 = NaiveDate::from_ymd_opt(2021, 12, 16)
+        .unwrap()
+        .and_hms_opt(0, 0, 0)
+        .unwrap();
+    let t1 = NaiveDate::from_ymd_opt(2021, 12, 16)
+        .unwrap()
+        .and_hms_opt(0, 30, 0)
+        .unwrap();
+    let t2 = NaiveDate::from_ymd_opt(2021, 12, 16)
+        .unwrap()
+        .and_hms_opt(1, 0, 0)
+        .unwrap();
     assert_eq!(
         g,
         &[
@@ -197,8 +220,14 @@ fn test_boundaries() {
             t2.timestamp_nanos()
         ]
     );
-    let b_start = NaiveDate::from_ymd(2021, 12, 16).and_hms(0, 0, 0);
-    let b_end = NaiveDate::from_ymd(2021, 12, 16).and_hms(1, 0, 0);
+    let b_start = NaiveDate::from_ymd_opt(2021, 12, 16)
+        .unwrap()
+        .and_hms_opt(0, 0, 0)
+        .unwrap();
+    let b_end = NaiveDate::from_ymd_opt(2021, 12, 16)
+        .unwrap()
+        .and_hms_opt(1, 0, 0)
+        .unwrap();
     assert_eq!(
         &[lower[0], higher[0]],
         &[b_start.timestamp_nanos(), b_end.timestamp_nanos()]
@@ -212,9 +241,18 @@ fn test_boundaries() {
     // 2021-12-16 01:30:00
     // 2021-12-16 02:00:00
     let g = take_groups_slice(&groups, 1, &ts);
-    let t0 = NaiveDate::from_ymd(2021, 12, 16).and_hms(1, 0, 0);
-    let t1 = NaiveDate::from_ymd(2021, 12, 16).and_hms(1, 30, 0);
-    let t2 = NaiveDate::from_ymd(2021, 12, 16).and_hms(2, 0, 0);
+    let t0 = NaiveDate::from_ymd_opt(2021, 12, 16)
+        .unwrap()
+        .and_hms_opt(1, 0, 0)
+        .unwrap();
+    let t1 = NaiveDate::from_ymd_opt(2021, 12, 16)
+        .unwrap()
+        .and_hms_opt(1, 30, 0)
+        .unwrap();
+    let t2 = NaiveDate::from_ymd_opt(2021, 12, 16)
+        .unwrap()
+        .and_hms_opt(2, 0, 0)
+        .unwrap();
     assert_eq!(
         g,
         &[
@@ -223,8 +261,14 @@ fn test_boundaries() {
             t2.timestamp_nanos()
         ]
     );
-    let b_start = NaiveDate::from_ymd(2021, 12, 16).and_hms(1, 0, 0);
-    let b_end = NaiveDate::from_ymd(2021, 12, 16).and_hms(2, 0, 0);
+    let b_start = NaiveDate::from_ymd_opt(2021, 12, 16)
+        .unwrap()
+        .and_hms_opt(1, 0, 0)
+        .unwrap();
+    let b_end = NaiveDate::from_ymd_opt(2021, 12, 16)
+        .unwrap()
+        .and_hms_opt(2, 0, 0)
+        .unwrap();
     assert_eq!(
         &[lower[1], higher[1]],
         &[b_start.timestamp_nanos(), b_end.timestamp_nanos()]
@@ -240,6 +284,7 @@ fn test_boundaries() {
         TimeUnit::Nanoseconds,
         false,
         false,
+        Default::default(),
     );
     assert_eq!(groups[0], [0, 2]); // 00:00:00 -> 00:30:00
     assert_eq!(groups[1], [2, 2]); // 01:00:00 -> 01:30:00
@@ -253,6 +298,7 @@ fn test_boundaries() {
         TimeUnit::Nanoseconds,
         false,
         false,
+        Default::default(),
     );
     assert_eq!(groups[0], [1, 2]); // 00:00:00 -> 00:30:00
     assert_eq!(groups[1], [3, 2]); // 01:00:00 -> 01:30:00
@@ -266,6 +312,7 @@ fn test_boundaries() {
         TimeUnit::Nanoseconds,
         false,
         false,
+        Default::default(),
     );
     assert_eq!(groups[0], [1, 1]); // 00:00:00 -> 00:30:00
     assert_eq!(groups[1], [3, 1]); // 01:00:00 -> 01:30:00
@@ -274,8 +321,14 @@ fn test_boundaries() {
 
 #[test]
 fn test_boundaries_2() {
-    let start = NaiveDate::from_ymd(2021, 12, 16).and_hms(0, 0, 0);
-    let stop = NaiveDate::from_ymd(2021, 12, 16).and_hms(4, 0, 0);
+    let start = NaiveDate::from_ymd_opt(2021, 12, 16)
+        .unwrap()
+        .and_hms_opt(0, 0, 0)
+        .unwrap();
+    let stop = NaiveDate::from_ymd_opt(2021, 12, 16)
+        .unwrap()
+        .and_hms_opt(4, 0, 0)
+        .unwrap();
 
     let ts = date_range_vec(
         start.timestamp_nanos(),
@@ -306,6 +359,7 @@ fn test_boundaries_2() {
         TimeUnit::Nanoseconds,
         true,
         true,
+        Default::default(),
     );
 
     // 1st group
@@ -317,11 +371,23 @@ fn test_boundaries_2() {
     // (note that we don't expect 01:30:00 because we close left (and thus open interval right))
     // see: https://pandas.pydata.org/docs/reference/api/pandas.Interval.html
     let g = take_groups_slice(&groups, 0, &ts);
-    let t0 = NaiveDate::from_ymd(2021, 12, 16).and_hms(0, 30, 0);
-    let t1 = NaiveDate::from_ymd(2021, 12, 16).and_hms(1, 0, 0);
+    let t0 = NaiveDate::from_ymd_opt(2021, 12, 16)
+        .unwrap()
+        .and_hms_opt(0, 30, 0)
+        .unwrap();
+    let t1 = NaiveDate::from_ymd_opt(2021, 12, 16)
+        .unwrap()
+        .and_hms_opt(1, 0, 0)
+        .unwrap();
     assert_eq!(g, &[t0.timestamp_nanos(), t1.timestamp_nanos()]);
-    let b_start = NaiveDate::from_ymd(2021, 12, 16).and_hms(0, 30, 0);
-    let b_end = NaiveDate::from_ymd(2021, 12, 16).and_hms(1, 30, 0);
+    let b_start = NaiveDate::from_ymd_opt(2021, 12, 16)
+        .unwrap()
+        .and_hms_opt(0, 30, 0)
+        .unwrap();
+    let b_end = NaiveDate::from_ymd_opt(2021, 12, 16)
+        .unwrap()
+        .and_hms_opt(1, 30, 0)
+        .unwrap();
     assert_eq!(
         &[lower[0], higher[0]],
         &[b_start.timestamp_nanos(), b_end.timestamp_nanos()]
@@ -335,11 +401,23 @@ fn test_boundaries_2() {
     // 2021-12-16 03:00:00
     // (note that we don't expect 03:30:00 because we close left)
     let g = take_groups_slice(&groups, 1, &ts);
-    let t0 = NaiveDate::from_ymd(2021, 12, 16).and_hms(2, 30, 0);
-    let t1 = NaiveDate::from_ymd(2021, 12, 16).and_hms(3, 0, 0);
+    let t0 = NaiveDate::from_ymd_opt(2021, 12, 16)
+        .unwrap()
+        .and_hms_opt(2, 30, 0)
+        .unwrap();
+    let t1 = NaiveDate::from_ymd_opt(2021, 12, 16)
+        .unwrap()
+        .and_hms_opt(3, 0, 0)
+        .unwrap();
     assert_eq!(g, &[t0.timestamp_nanos(), t1.timestamp_nanos()]);
-    let b_start = NaiveDate::from_ymd(2021, 12, 16).and_hms(2, 30, 0);
-    let b_end = NaiveDate::from_ymd(2021, 12, 16).and_hms(3, 30, 0);
+    let b_start = NaiveDate::from_ymd_opt(2021, 12, 16)
+        .unwrap()
+        .and_hms_opt(2, 30, 0)
+        .unwrap();
+    let b_end = NaiveDate::from_ymd_opt(2021, 12, 16)
+        .unwrap()
+        .and_hms_opt(3, 30, 0)
+        .unwrap();
     assert_eq!(
         &[lower[1], higher[1]],
         &[b_start.timestamp_nanos(), b_end.timestamp_nanos()]
@@ -348,8 +426,14 @@ fn test_boundaries_2() {
 
 #[test]
 fn test_boundaries_ms() {
-    let start = NaiveDate::from_ymd(2021, 12, 16).and_hms(0, 0, 0);
-    let stop = NaiveDate::from_ymd(2021, 12, 16).and_hms(3, 0, 0);
+    let start = NaiveDate::from_ymd_opt(2021, 12, 16)
+        .unwrap()
+        .and_hms_opt(0, 0, 0)
+        .unwrap();
+    let stop = NaiveDate::from_ymd_opt(2021, 12, 16)
+        .unwrap()
+        .and_hms_opt(3, 0, 0)
+        .unwrap();
 
     let ts = date_range_vec(
         start.timestamp_millis(),
@@ -380,6 +464,7 @@ fn test_boundaries_ms() {
         TimeUnit::Milliseconds,
         true,
         true,
+        Default::default(),
     );
 
     // 1st group
@@ -390,9 +475,18 @@ fn test_boundaries_ms() {
     // 2021-12-16 00:30:00
     // 2021-12-16 01:00:00
     let g = take_groups_slice(&groups, 0, &ts);
-    let t0 = NaiveDate::from_ymd(2021, 12, 16).and_hms(0, 0, 0);
-    let t1 = NaiveDate::from_ymd(2021, 12, 16).and_hms(0, 30, 0);
-    let t2 = NaiveDate::from_ymd(2021, 12, 16).and_hms(1, 0, 0);
+    let t0 = NaiveDate::from_ymd_opt(2021, 12, 16)
+        .unwrap()
+        .and_hms_opt(0, 0, 0)
+        .unwrap();
+    let t1 = NaiveDate::from_ymd_opt(2021, 12, 16)
+        .unwrap()
+        .and_hms_opt(0, 30, 0)
+        .unwrap();
+    let t2 = NaiveDate::from_ymd_opt(2021, 12, 16)
+        .unwrap()
+        .and_hms_opt(1, 0, 0)
+        .unwrap();
     assert_eq!(
         g,
         &[
@@ -401,8 +495,14 @@ fn test_boundaries_ms() {
             t2.timestamp_millis()
         ]
     );
-    let b_start = NaiveDate::from_ymd(2021, 12, 16).and_hms(0, 0, 0);
-    let b_end = NaiveDate::from_ymd(2021, 12, 16).and_hms(1, 0, 0);
+    let b_start = NaiveDate::from_ymd_opt(2021, 12, 16)
+        .unwrap()
+        .and_hms_opt(0, 0, 0)
+        .unwrap();
+    let b_end = NaiveDate::from_ymd_opt(2021, 12, 16)
+        .unwrap()
+        .and_hms_opt(1, 0, 0)
+        .unwrap();
     assert_eq!(
         &[lower[0], higher[0]],
         &[b_start.timestamp_millis(), b_end.timestamp_millis()]
@@ -416,9 +516,18 @@ fn test_boundaries_ms() {
     // 2021-12-16 01:30:00
     // 2021-12-16 02:00:00
     let g = take_groups_slice(&groups, 1, &ts);
-    let t0 = NaiveDate::from_ymd(2021, 12, 16).and_hms(1, 0, 0);
-    let t1 = NaiveDate::from_ymd(2021, 12, 16).and_hms(1, 30, 0);
-    let t2 = NaiveDate::from_ymd(2021, 12, 16).and_hms(2, 0, 0);
+    let t0 = NaiveDate::from_ymd_opt(2021, 12, 16)
+        .unwrap()
+        .and_hms_opt(1, 0, 0)
+        .unwrap();
+    let t1 = NaiveDate::from_ymd_opt(2021, 12, 16)
+        .unwrap()
+        .and_hms_opt(1, 30, 0)
+        .unwrap();
+    let t2 = NaiveDate::from_ymd_opt(2021, 12, 16)
+        .unwrap()
+        .and_hms_opt(2, 0, 0)
+        .unwrap();
     assert_eq!(
         g,
         &[
@@ -427,8 +536,14 @@ fn test_boundaries_ms() {
             t2.timestamp_millis()
         ]
     );
-    let b_start = NaiveDate::from_ymd(2021, 12, 16).and_hms(1, 0, 0);
-    let b_end = NaiveDate::from_ymd(2021, 12, 16).and_hms(2, 0, 0);
+    let b_start = NaiveDate::from_ymd_opt(2021, 12, 16)
+        .unwrap()
+        .and_hms_opt(1, 0, 0)
+        .unwrap();
+    let b_end = NaiveDate::from_ymd_opt(2021, 12, 16)
+        .unwrap()
+        .and_hms_opt(2, 0, 0)
+        .unwrap();
     assert_eq!(
         &[lower[1], higher[1]],
         &[b_start.timestamp_millis(), b_end.timestamp_millis()]
@@ -444,6 +559,7 @@ fn test_boundaries_ms() {
         TimeUnit::Milliseconds,
         false,
         false,
+        Default::default(),
     );
     assert_eq!(groups[0], [0, 2]); // 00:00:00 -> 00:30:00
     assert_eq!(groups[1], [2, 2]); // 01:00:00 -> 01:30:00
@@ -457,6 +573,7 @@ fn test_boundaries_ms() {
         TimeUnit::Milliseconds,
         false,
         false,
+        Default::default(),
     );
     assert_eq!(groups[0], [1, 2]); // 00:00:00 -> 00:30:00
     assert_eq!(groups[1], [3, 2]); // 01:00:00 -> 01:30:00
@@ -470,6 +587,7 @@ fn test_boundaries_ms() {
         TimeUnit::Milliseconds,
         false,
         false,
+        Default::default(),
     );
     assert_eq!(groups[0], [1, 1]); // 00:00:00 -> 00:30:00
     assert_eq!(groups[1], [3, 1]); // 01:00:00 -> 01:30:00
@@ -479,8 +597,14 @@ fn test_boundaries_ms() {
 #[test]
 fn test_rolling_lookback() {
     // Test month as interval in date range
-    let start = NaiveDate::from_ymd(1970, 1, 16).and_hms(0, 0, 0);
-    let end = NaiveDate::from_ymd(1970, 1, 16).and_hms(4, 0, 0);
+    let start = NaiveDate::from_ymd_opt(1970, 1, 16)
+        .unwrap()
+        .and_hms_opt(0, 0, 0)
+        .unwrap();
+    let end = NaiveDate::from_ymd_opt(1970, 1, 16)
+        .unwrap()
+        .and_hms_opt(4, 0, 0)
+        .unwrap();
     let dates = date_range_vec(
         start.timestamp_millis(),
         end.timestamp_millis(),
@@ -574,11 +698,15 @@ fn test_rolling_lookback() {
 #[test]
 fn test_end_membership() {
     let time = [
-        NaiveDate::from_ymd(2021, 2, 1)
-            .and_hms(0, 0, 0)
+        NaiveDate::from_ymd_opt(2021, 2, 1)
+            .unwrap()
+            .and_hms_opt(0, 0, 0)
+            .unwrap()
             .timestamp_millis(),
-        NaiveDate::from_ymd(2021, 5, 1)
-            .and_hms(0, 0, 0)
+        NaiveDate::from_ymd_opt(2021, 5, 1)
+            .unwrap()
+            .and_hms_opt(0, 0, 0)
+            .unwrap()
             .timestamp_millis(),
     ];
     let window = Window::new(
@@ -600,6 +728,7 @@ fn test_end_membership() {
         TimeUnit::Milliseconds,
         false,
         false,
+        Default::default(),
     );
     assert_eq!(groups[0], [0, 1]);
     assert_eq!(groups[1], [0, 1]);
@@ -622,6 +751,7 @@ fn test_groupby_windows_membership_2791() {
         TimeUnit::Milliseconds,
         false,
         false,
+        Default::default(),
     );
     assert_eq!(groups[0], [0, 2]);
     assert_eq!(groups[1], [2, 2]);
@@ -643,6 +773,7 @@ fn test_groupby_windows_duplicates_2931() {
         TimeUnit::Milliseconds,
         false,
         false,
+        Default::default(),
     );
     assert_eq!(groups, [[0, 1], [1, 2], [3, 2]]);
 }
@@ -650,13 +781,13 @@ fn test_groupby_windows_duplicates_2931() {
 #[test]
 fn test_groupby_windows_offsets_3776() {
     let dates = &[
-        NaiveDate::from_ymd(2020, 12, 1),
-        NaiveDate::from_ymd(2021, 2, 1),
-        NaiveDate::from_ymd(2021, 5, 1),
+        NaiveDate::from_ymd_opt(2020, 12, 1).unwrap(),
+        NaiveDate::from_ymd_opt(2021, 2, 1).unwrap(),
+        NaiveDate::from_ymd_opt(2021, 5, 1).unwrap(),
     ];
     let ts = dates
         .iter()
-        .map(|d| d.and_hms(0, 0, 0).timestamp_nanos())
+        .map(|d| d.and_hms_opt(0, 0, 0).unwrap().timestamp_nanos())
         .collect::<Vec<_>>();
 
     let window = Window::new(
@@ -671,6 +802,7 @@ fn test_groupby_windows_offsets_3776() {
         TimeUnit::Nanoseconds,
         false,
         false,
+        Default::default(),
     );
     assert_eq!(groups, [[0, 1], [1, 1], [2, 1]]);
 }
