@@ -1,20 +1,28 @@
 from __future__ import annotations
 
+import contextlib
 import json
 import os
-import sys
-from types import TracebackType
+from typing import TYPE_CHECKING
 
-if sys.version_info >= (3, 8):
-    from typing import Literal
-else:
-    from typing_extensions import Literal
+with contextlib.suppress(ImportError):  # Module not available when building docs
+    from polars.polars import set_float_fmt as _set_float_fmt
 
 
+if TYPE_CHECKING:
+    import sys
+    from types import TracebackType
+
+    if sys.version_info >= (3, 8):
+        from typing import Literal
+    else:
+        from typing_extensions import Literal
+
 # note: register all Config-specific environment variable names here; need to constrain
 # which 'POLARS_' environment variables are recognised, as there are other lower-level
 # and/or experimental settings that should not be saved or reset with the Config vars.
 POLARS_CFG_ENV_VARS = {
+    "POLARS_AUTO_STRUCTIFY",
     "POLARS_FMT_MAX_COLS",
     "POLARS_FMT_MAX_ROWS",
     "POLARS_FMT_STR_LEN",
@@ -26,11 +34,11 @@ POLARS_CFG_ENV_VARS = {
     "POLARS_FMT_TABLE_HIDE_COLUMN_SEPARATOR",
     "POLARS_FMT_TABLE_HIDE_DATAFRAME_SHAPE_INFORMATION",
     "POLARS_FMT_TABLE_INLINE_COLUMN_DATA_TYPE",
+    "POLARS_FMT_TABLE_ROUNDED_CORNERS",
     "POLARS_TABLE_WIDTH",
     "POLARS_VERBOSE",
+    "POLARS_ACTIVATE_DECIMAL",
 }
-# register Config-local attributes (with their defaults) here
-POLARS_CFG_LOCAL_VARS = {"with_columns_kwargs": False}
 
 
 class Config:
@@ -64,10 +72,6 @@ class Config:
         """Reset any Config options that were set within the scope."""
         self.restore_defaults().load(self._original_state)
 
-    # note: class-local attributes can be used for options that don't have
-    # a Rust component (so, no need to register environment variables).
-    with_columns_kwargs: bool = False
-
     @classmethod
     def load(cls, cfg: str) -> type[Config]:
         """
@@ -94,7 +98,7 @@ class Config:
         Notes
         -----
         This method operates by removing all Config options from the environment,
-        and then setting any class-local flags back to their default value.
+        and then setting any local (non-env) options back to their default value.
 
         Examples
         --------
@@ -103,8 +107,7 @@ class Config:
         """
         for var in POLARS_CFG_ENV_VARS:
             os.environ.pop(var, None)
-        for flag, value in POLARS_CFG_LOCAL_VARS.items():
-            setattr(cls, flag, value)
+        cls.set_fmt_float()
         return cls
 
     @classmethod
@@ -122,9 +125,10 @@ class Config:
             for key in sorted(POLARS_CFG_ENV_VARS)
             if (key in os.environ)
         }
-        config_vars = {attr: getattr(cls, attr) for attr in POLARS_CFG_LOCAL_VARS}
+        # note: 'local' vars are unused; preserved here for
+        # backwards-compatibility with previously-saved configs
         return json.dumps(
-            {"environment": environment_vars, "local": config_vars},
+            {"environment": environment_vars, "local": {}},
             separators=(",", ":"),
         )
 
@@ -169,17 +173,21 @@ class Config:
         # │ f64 ┆ bool  │      | f64 | bool  |
         # ╞═════╪═══════╡      +=============+
         # │ 1.0 ┆ true  │  >>  | 1.0 | true  |
-        # ├╌╌╌╌╌┼╌╌╌╌╌╌╌┤      |-----+-------|
         # │ 2.5 ┆ false │      | 2.5 | false |
-        # ├╌╌╌╌╌┼╌╌╌╌╌╌╌┤      |-----+-------|
         # │ 5.0 ┆ true  │      | 5.0 | true  |
         # └─────┴───────┘      +-----+-------+
 
         """
-        fmt = "ASCII_FULL" if active else "UTF8_FULL"
+        fmt = "ASCII_FULL_CONDENSED" if active else "UTF8_FULL_CONDENSED"
         os.environ["POLARS_FMT_TABLE_FORMATTING"] = fmt
         return cls
 
+    @classmethod
+    def set_auto_structify(cls, active: bool = False) -> type[Config]:
+        """Allow multi-output expressions to be automatically turned into Structs."""
+        os.environ["POLARS_AUTO_STRUCTIFY"] = str(int(active))
+        return cls
+
     @classmethod
     def set_fmt_str_lengths(cls, n: int) -> type[Config]:
         """
@@ -222,9 +230,7 @@ class Config:
         # │        f64 ┆       bool │
         # ╞════════════╪════════════╡
         # │        1.0 ┆       true │
-        # ├╌╌╌╌╌╌╌╌╌╌╌╌┼╌╌╌╌╌╌╌╌╌╌╌╌┤
         # │        2.5 ┆      false │
-        # ├╌╌╌╌╌╌╌╌╌╌╌╌┼╌╌╌╌╌╌╌╌╌╌╌╌┤
         # │        5.0 ┆       true │
         # └────────────┴────────────┘
 
@@ -263,7 +269,7 @@ class Config:
         └─────┴─────┴─────┴─────┴─────┴─────┘
 
         >>> with pl.Config() as cfg:
-        ...     pl.cfg.Config.set_tbl_cols(10)  # doctest: +IGNORE_RESULT
+        ...     cfg.set_tbl_cols(10)  # doctest: +IGNORE_RESULT
         ...     df
         ...
         shape: (1, 100)
@@ -294,11 +300,9 @@ class Config:
         # │ abc ┆ xyz   │      │ abc (f64) ┆ xyz (bool) │
         # │ --- ┆ ---   │      ╞═══════════╪════════════╡
         # │ f64 ┆ bool  │      │ 1.0       ┆ true       │
-        # ╞═════╪═══════╡      ├╌╌╌╌╌╌╌╌╌╌╌┼╌╌╌╌╌╌╌╌╌╌╌╌┤
-        # │ 1.0 ┆ true  │  >>  │ 2.5       ┆ false      │
-        # ├╌╌╌╌╌┼╌╌╌╌╌╌╌┤      ├╌╌╌╌╌╌╌╌╌╌╌┼╌╌╌╌╌╌╌╌╌╌╌╌┤
-        # │ 2.5 ┆ false │      │ 5.0       ┆ true       │
-        # ├╌╌╌╌╌┼╌╌╌╌╌╌╌┤      └───────────┴────────────┘
+        # ╞═════╪═══════╡  >>  │ 2.5       ┆ false      │
+        # │ 1.0 ┆ true  │      │ 5.0       ┆ true       │
+        # │ 2.5 ┆ false │      └───────────┴────────────┘
         # │ 5.0 ┆ true  │
         # └─────┴───────┘
 
@@ -321,11 +325,9 @@ class Config:
         # │ abc ┆ xyz   │      │ --- ┆ ---   │
         # │ --- ┆ ---   │      │ f64 ┆ bool  │
         # │ f64 ┆ bool  │      ╞═════╪═══════╡
-        # ╞═════╪═══════╡      │ 1.0 ┆ true  │
-        # │ 1.0 ┆ true  │  >>  ├╌╌╌╌╌┼╌╌╌╌╌╌╌┤
-        # ├╌╌╌╌╌┼╌╌╌╌╌╌╌┤      │ 2.5 ┆ false │
-        # │ 2.5 ┆ false │      ├╌╌╌╌╌┼╌╌╌╌╌╌╌┤
-        # ├╌╌╌╌╌┼╌╌╌╌╌╌╌┤      │ 5.0 ┆ true  │
+        # ╞═════╪═══════╡  >>  │ 1.0 ┆ true  │
+        # │ 1.0 ┆ true  │      │ 2.5 ┆ false │
+        # │ 2.5 ┆ false │      │ 5.0 ┆ true  │
         # │ 5.0 ┆ true  │      └─────┴───────┘
         # └─────┴───────┘      shape: (3, 2)
 
@@ -336,20 +338,25 @@ class Config:
     @classmethod
     def set_tbl_formatting(
         cls,
-        format: Literal[
-            "ASCII_FULL",
-            "ASCII_NO_BORDERS",
-            "ASCII_BORDERS_ONLY",
-            "ASCII_BORDERS_ONLY_CONDENSED",
-            "ASCII_HORIZONTAL_ONLY",
-            "ASCII_MARKDOWN",
-            "UTF8_FULL",
-            "UTF8_FULL_CONDENSED",
-            "UTF8_NO_BORDERS",
-            "UTF8_BORDERS_ONLY",
-            "UTF8_HORIZONTAL_ONLY",
-            "NOTHING",
-        ],
+        format: (
+            Literal[
+                "ASCII_FULL",
+                "ASCII_FULL_CONDENSED",
+                "ASCII_NO_BORDERS",
+                "ASCII_BORDERS_ONLY",
+                "ASCII_BORDERS_ONLY_CONDENSED",
+                "ASCII_HORIZONTAL_ONLY",
+                "ASCII_MARKDOWN",
+                "UTF8_FULL",
+                "UTF8_FULL_CONDENSED",
+                "UTF8_NO_BORDERS",
+                "UTF8_BORDERS_ONLY",
+                "UTF8_HORIZONTAL_ONLY",
+                "NOTHING",
+            ]
+            | None
+        ) = None,
+        rounded_corners: bool = False,
     ) -> type[Config]:
         """
         Set table formatting style.
@@ -357,19 +364,23 @@ class Config:
         Parameters
         ----------
         format : str
-            * "ASCII_FULL": ASCII, borders / lines.
+            * "ASCII_FULL": ASCII, with all borders and lines, including row dividers.
+            * "ASCII_FULL_CONDENSED": Same as ASCII_FULL, but with dense row spacing.
             * "ASCII_NO_BORDERS": ASCII, no borders.
             * "ASCII_BORDERS_ONLY": ASCII, borders only.
             * "ASCII_BORDERS_ONLY_CONDENSED": ASCII, borders only, dense row spacing.
             * "ASCII_HORIZONTAL_ONLY": ASCII, horizontal lines only.
             * "ASCII_MARKDOWN": ASCII, Markdown compatible.
-            * "UTF8_FULL": UTF8, with all borders and lines (default).
-            * "UTF8_FULL_CONDENSED": Same as UTF8_FULL, with dense row spacing.
+            * "UTF8_FULL": UTF8, with all borders and lines, including row dividers.
+            * "UTF8_FULL_CONDENSED": Same as UTF8_FULL, but with dense row spacing.
             * "UTF8_NO_BORDERS": UTF8, no borders.
             * "UTF8_BORDERS_ONLY": UTF8, borders only.
             * "UTF8_HORIZONTAL_ONLY": UTF8, horizontal lines only.
             * "NOTHING": No borders or other lines.
 
+        rounded_corners : bool
+            apply rounded corners to UTF8-styled tables (no-op for ASCII formats).
+
         Notes
         -----
         The UTF8 styles all use one or more of the semigraphic box-drawing characters
@@ -383,7 +394,9 @@ class Config:
         """
         # can see what the different styles look like in the comfy-table tests:
         # https://github.com/Nukesor/comfy-table/blob/main/tests/all/presets_test.rs
-        os.environ["POLARS_FMT_TABLE_FORMATTING"] = format
+        if format:
+            os.environ["POLARS_FMT_TABLE_FORMATTING"] = format
+        os.environ["POLARS_FMT_TABLE_ROUNDED_CORNERS"] = str(int(rounded_corners))
         return cls
 
     @classmethod
@@ -401,11 +414,9 @@ class Config:
         # │ abc ┆ xyz   │      │ abc ┆ xyz   │
         # │ --- ┆ ---   │      ╞═════╪═══════╡
         # │ f64 ┆ bool  │      │ 1.0 ┆ true  │
-        # ╞═════╪═══════╡      ├╌╌╌╌╌┼╌╌╌╌╌╌╌┤
-        # │ 1.0 ┆ true  │  >>  │ 2.5 ┆ false │
-        # ├╌╌╌╌╌┼╌╌╌╌╌╌╌┤      ├╌╌╌╌╌┼╌╌╌╌╌╌╌┤
-        # │ 2.5 ┆ false │      │ 5.0 ┆ true  │
-        # ├╌╌╌╌╌┼╌╌╌╌╌╌╌┤      └─────┴───────┘
+        # ╞═════╪═══════╡  >>  │ 2.5 ┆ false │
+        # │ 1.0 ┆ true  │      │ 5.0 ┆ true  │
+        # │ 2.5 ┆ false │      └─────┴───────┘
         # │ 5.0 ┆ true  │
         # └─────┴───────┘
 
@@ -428,11 +439,9 @@ class Config:
         # │ abc ┆ xyz   │      │ f64 ┆ bool  │
         # │ --- ┆ ---   │      ╞═════╪═══════╡
         # │ f64 ┆ bool  │      │ 1.0 ┆ true  │
-        # ╞═════╪═══════╡      ├╌╌╌╌╌┼╌╌╌╌╌╌╌┤
-        # │ 1.0 ┆ true  │  >>  │ 2.5 ┆ false │
-        # ├╌╌╌╌╌┼╌╌╌╌╌╌╌┤      ├╌╌╌╌╌┼╌╌╌╌╌╌╌┤
-        # │ 2.5 ┆ false │      │ 5.0 ┆ true  │
-        # ├╌╌╌╌╌┼╌╌╌╌╌╌╌┤      └─────┴───────┘
+        # ╞═════╪═══════╡  >>  │ 2.5 ┆ false │
+        # │ 1.0 ┆ true  │      │ 5.0 ┆ true  │
+        # │ 2.5 ┆ false │      └─────┴───────┘
         # │ 5.0 ┆ true  │
         # └─────┴───────┘
 
@@ -456,10 +465,8 @@ class Config:
         # │ --- ┆ ---   │      │ f64 ┆ bool  │
         # │ f64 ┆ bool  │      ╞═════╪═══════╡
         # ╞═════╪═══════╡      │ 1.0 ┆ true  │
-        # │ 1.0 ┆ true  │  >>  ├╌╌╌╌╌┼╌╌╌╌╌╌╌┤
-        # ├╌╌╌╌╌┼╌╌╌╌╌╌╌┤      │ 2.5 ┆ false │
-        # │ 2.5 ┆ false │      ├╌╌╌╌╌┼╌╌╌╌╌╌╌┤
-        # ├╌╌╌╌╌┼╌╌╌╌╌╌╌┤      │ 5.0 ┆ true  │
+        # │ 1.0 ┆ true  │  >>  │ 2.5 ┆ false │
+        # │ 2.5 ┆ false │      │ 5.0 ┆ true  │
         # │ 5.0 ┆ true  │      └─────┴───────┘
         # └─────┴───────┘
 
@@ -487,10 +494,8 @@ class Config:
         # │ --- ┆ ---   │      │ f64 ┆ bool  │
         # │ f64 ┆ bool  │      ╞═════╪═══════╡
         # ╞═════╪═══════╡      │ 1.0 ┆ true  │
-        # │ 1.0 ┆ true  │  >>  ├╌╌╌╌╌┼╌╌╌╌╌╌╌┤
-        # ├╌╌╌╌╌┼╌╌╌╌╌╌╌┤      │ 2.5 ┆ false │
-        # │ 2.5 ┆ false │      ├╌╌╌╌╌┼╌╌╌╌╌╌╌┤
-        # ├╌╌╌╌╌┼╌╌╌╌╌╌╌┤      │ 5.0 ┆ true  │
+        # │ 1.0 ┆ true  │  >>  │ 2.5 ┆ false │
+        # │ 2.5 ┆ false │      │ 5.0 ┆ true  │
         # │ 5.0 ┆ true  │      └─────┴───────┘
         # └─────┴───────┘
 
@@ -525,9 +530,7 @@ class Config:
         # │ f64 ┆ bool  │
         # ╞═════╪═══════╡
         # │ 1.0 ┆ true  │
-        # ├╌╌╌╌╌┼╌╌╌╌╌╌╌┤
         # │ ... ┆ ...   │
-        # ├╌╌╌╌╌┼╌╌╌╌╌╌╌┤
         # │ 5.0 ┆ false │
         # └─────┴───────┘
 
@@ -554,3 +557,32 @@ class Config:
         """Enable additional verbose/debug logging."""
         os.environ["POLARS_VERBOSE"] = str(int(active))
         return cls
+
+    @classmethod
+    def set_fmt_float(cls, fmt: str = "mixed") -> type[Config]:
+        """
+        Control how floating  point values are displayed.
+
+        Parameters
+        ----------
+        fmt : {"mixed", "full"}
+            How to format floating point numbers
+
+        """
+        _set_float_fmt(fmt)
+        return cls
+
+    @classmethod
+    def activate_decimals(cls) -> type[Config]:
+        """
+        Activate ``Decimal`` data types.
+
+        This is temporary setting that will be removed later once
+        ``Decimal`` type stabilize. This happens without it being
+        considered a breaking change.
+
+        Currently, ``Decimal`` types are in alpha stage.
+
+        """
+        os.environ["POLARS_ACTIVATE_DECIMAL"] = "1"
+        return cls
