@@ -1,16 +1,14 @@
 from __future__ import annotations
 
+import contextlib
 from pathlib import Path
-from typing import Sequence
+from typing import TYPE_CHECKING, Sequence
 
 import polars.internals as pli
 from polars.datatypes import (
     N_INFER_DEFAULT,
-    PolarsDataType,
-    SchemaDict,
     py_type_to_dtype,
 )
-from polars.internals.type_aliases import CsvEncoding
 from polars.utils import (
     _prepare_row_count_args,
     _process_null_values,
@@ -18,12 +16,12 @@ from polars.utils import (
     normalise_filepath,
 )
 
-try:
+with contextlib.suppress(ImportError):  # Module not available when building docs
     from polars.polars import PyBatchedCsv
 
-    _DOCUMENTING = False
-except ImportError:
-    _DOCUMENTING = True
+if TYPE_CHECKING:
+    from polars.datatypes import PolarsDataType, SchemaDict
+    from polars.internals.type_aliases import CsvEncoding
 
 
 class BatchedCsvReader:
