@@ -15,3 +15,10 @@ def test_top_k_empty() -> None:
     df = pl.DataFrame({"test": []})
 
     assert_frame_equal(df.select([pl.col("test").top_k(2)]), df)
+
+
+def test_empty_cross_join() -> None:
+    a = pl.LazyFrame(schema={"a": pl.Int32})
+    b = pl.LazyFrame(schema={"b": pl.Int32})
+
+    assert (a.join(b, how="cross").collect()).schema == {"a": pl.Int32, "b": pl.Int32}
